@@ -1,6 +1,6 @@
 -- 
 -- Created by SQL::Translator::Producer::SQLite
--- Created on Sat Jun 27 14:02:39 2009
+-- Created on Wed Aug 12 16:10:43 2009
 -- 
 
 
@@ -63,7 +63,8 @@ CREATE TABLE event (
   created_on timestamp NOT NULL,
   varchar_date varchar(20),
   varchar_datetime varchar(20),
-  skip_inflation datetime
+  skip_inflation datetime,
+  ts_without_tz datetime
 );
 
 --
@@ -106,6 +107,14 @@ CREATE TABLE link (
   title varchar(100)
 );
 
+--
+-- Table: money_test
+--
+CREATE TABLE money_test (
+  id INTEGER PRIMARY KEY NOT NULL,
+  amount money
+);
+
 --
 -- Table: noprimarykey
 --
@@ -225,7 +234,7 @@ CREATE INDEX artist_undirected_map_idx_id2 ON artist_undirected_map (id2);
 --
 CREATE TABLE bookmark (
   id INTEGER PRIMARY KEY NOT NULL,
-  link integer NOT NULL
+  link integer
 );
 
 CREATE INDEX bookmark_idx_link ON bookmark (link);
@@ -275,7 +284,8 @@ CREATE TABLE track (
   position integer NOT NULL,
   title varchar(100) NOT NULL,
   last_updated_on datetime,
-  last_updated_at datetime
+  last_updated_at datetime,
+  small_dt smalldatetime
 );
 
 CREATE INDEX track_idx_cd ON track (cd);
